#!/bin/bash

# kernel must be adjusted
# echo '|<path to script> %e %E %p %s %t' > /proc/sys/kernel/core_pattern
# parameters with example
# %e executable filename (without path prefix), callweaver
# %E  pathname of executable, !usr!sbin!callweaver
# %p  PID of dumped process, 3791
# %s  number of signal causing dump, 11
# %t  time of dump, expressed as seconds since the Epoch, 1538560002
# http://man7.org/linux/man-pages/man5/core.5.html

# list of executable names for dumps
CATCH="callweaver rtpengine kamailio tproxyd turnserver smsd whoteld"

APP=$(basename $0)
WORKPATH='/tmp/cddata'
MOUNT='/dev/shm' # mount point for checking free space
LIMIT=100 # MB
EXTRADIR='/opt/cd-wildix' # path to extensions, please read README there

log () {
  logger -p syslog.info -t $APP "$1"
}

# execute extensions from specific directory $1
run_extra () {
  if [ -d $1 ]; then
    for extrascript in $(ls -A $1); do
      if [ -x $1/$extrascript ]; then
        $1/$extrascript $2 $3 $4 $5 > /dev/null 2>&1
      fi
    done
  fi
}

# get package version from the file path
get_version () {
  local qresult=$(dpkg-query -S $1 2> /dev/null)
  if [ -n "$qresult" ]; then
    local package=$(echo $qresult | sed -e 's/://' | awk '{print $1}')
    echo -n $(dpkg-query -W -f='${Version}' $package)
  else
    echo -n "unknown"
  fi
}

# get build id
get_build () {
  eu-readelf -n $1 | sed -n 's/.*Build ID: \([0-9a-f]\)/\1/p'
}

FILENAME=$1
TIMESTAMP=$(date --date=@$5 --rfc-3339=seconds)
SIG_NUM=$4

log "Executable $FILENAME crashed at time $TIMESTAMP with signal $SIG_NUM"

# save dumps only for specific executable names
# they are defined in the top of file
if [ -z "$(grep -w $FILENAME <<< $CATCH)" ]; then
    log "Executable $FILENAME not in catch list, exiting"
    exit 0
fi

mkdir -p $WORKPATH

EXISTING=$(ls -A $WORKPATH)
if [ -n "$EXISTING" ]; then
    log "There is file $EXISTING for sending, exiting"
    exit 0
fi

FREE=$(df --block-size=1048576 --out=target --output=avail | grep $MOUNT | awk '{print $2}')

if ((FREE < LIMIT)); then
    log "${FREE}MB left on device ${MOUNT}, limit is ${LIMIT}MB, exiting"
    exit 0
fi

PATHNAME=$(sed -e 's/!/\//g' <<< $2)
PROCPID=$3

# important note: need to run extensions before stdin reading, otherwise kernel does most of cleaning
mkdir -p $WORKPATH/extra
# run common extensions
run_extra $EXTRADIR $FILENAME $PATHNAME $PROCPID $WORKPATH/extra
# run specific extensions
run_extra $EXTRADIR/$FILENAME $FILENAME $PATHNAME $PROCPID $WORKPATH/extra

NAMENS=$(date +%N)
ARCHIVE="${WORKPATH}/${FILENAME}-${NAMENS}.zip"

# save and rename the dump
cat /dev/stdin | zip -q -1 $ARCHIVE -
echo -ne "@ -\n@=core\n" | zipnote -w $ARCHIVE

# needed for proper subdir into zip
cd $WORKPATH
if [ -d /lib/x86_64-linux-gnu ]; then
    cp /lib/x86_64-linux-gnu/libc-*.so  libc.so.6
else
    cp /lib/i386-linux-gnu/libc-*.so  libc.so.6
fi  
zip -grqm -1 $ARCHIVE libc.so.6
rm -f libc.so.6
zip -grqm -1 $ARCHIVE extra

VERSION=$(get_version $PATHNAME)
BUILD=$(get_build $PATHNAME)

JSON="{\n  \"process\": \"$FILENAME\",\n  \"path\": \"$PATHNAME\",\n  \"signal\": $SIG_NUM,\n  \"timestamp\": \"$TIMESTAMP\",\n  \"created\": \"$(date --rfc-3339=seconds)\",\n  \"version\": \"$VERSION\",\n  \"build_id\": \"$BUILD\"\n}"
echo -e "$JSON" > ${WORKPATH}/core.json
zip -gjqm -1 $ARCHIVE ${WORKPATH}/core.json

chown wms:wms $ARCHIVE
log "Dump archive $ARCHIVE created"

/usr/sbin/send_crash_dump $ARCHIVE > /dev/null 2>&1 &

exit 0
